package org.playground.base.aspects;

import org.playground.base.TenantEntityManagerProvider;
import org.playground.base.annotations.TenantReadTransaction;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

@Aspect
@Component
public class TenantReadTransactionAspect {
	private static final Logger LOGGER = LoggerFactory.getLogger(TenantReadTransactionAspect.class);

	private final TenantEntityManagerProvider tenantEntityManagerProvider;

	public TenantReadTransactionAspect(TenantEntityManagerProvider tenantEntityManagerProvider) {
		this.tenantEntityManagerProvider = tenantEntityManagerProvider;
	}

	@Around("@annotation(tenantReadTransaction)")
	public Object tenantTransaction(ProceedingJoinPoint joinPoint, TenantReadTransaction tenantReadTransaction) throws Throwable {
		Object proceed;

		LOGGER.debug("Starting transaction for: {}", joinPoint.getSignature().getName());
		this.tenantEntityManagerProvider.beginTransaction();
		proceed = joinPoint.proceed();
		this.tenantEntityManagerProvider.closeTransaction();

		return proceed;
	}
}
