package org.playground.base.aspects;

import org.playground.base.TenantEntityManagerProvider;
import org.playground.base.annotations.TenantWriteTransaction;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.persistence.PersistenceException;

@Aspect
@Component
public class TenantWriteTransactionAspect {
	private final Logger LOGGER = LoggerFactory.getLogger(TenantWriteTransactionAspect.class);

	private final TenantEntityManagerProvider tenantEntityManagerProvider;

	public TenantWriteTransactionAspect(TenantEntityManagerProvider tenantEntityManagerProvider) {
		this.tenantEntityManagerProvider = tenantEntityManagerProvider;
	}

	@Around("@annotation(tenantWriteTransaction)")
	public Object tenantTransaction(ProceedingJoinPoint joinPoint, TenantWriteTransaction tenantWriteTransaction) throws Throwable {
		Object proceed = null;
		try {
			LOGGER.debug("Starting transaction for: {}", joinPoint.getSignature().getName());
			this.tenantEntityManagerProvider.beginTransaction();
			proceed = joinPoint.proceed();
			this.tenantEntityManagerProvider.commitTransaction();
		} catch(PersistenceException e) {
			LOGGER.error("Rollback transaction for: {}", joinPoint.getSignature().getName(), e);
			this.tenantEntityManagerProvider.rollbackTransaction();
		} catch(Exception e) {
			LOGGER.error("Rollback transaction for: {}", joinPoint.getSignature().getName(), e);
			this.tenantEntityManagerProvider.rollbackTransaction();
			throw e;
		}

		return proceed;
	}
}
