package org.playground.base.models;

import org.playground.base.models.embeddable.CostObject;
import org.playground.base.models.superClasses.BaseCostElement;
import org.apache.commons.collections4.CollectionUtils;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.List;

@Entity
public class Element extends BaseCostElement {
	@OneToMany(mappedBy = "parent", fetch = FetchType.EAGER)
	private List<Element> children;

	@ManyToOne
	private Element parent;

	@OneToOne
	private ElementType type;

	public static Element of(ElementType elementType, Element parent) {
		Element newElement = new Element();
		newElement.setParent(parent);
		newElement.setType(elementType);
		return newElement;
	}

	public List<CostObject> getAllCostObjects() {
		ArrayList<CostObject> result = new ArrayList<>(this.getCostObjects());
		if(CollectionUtils.isNotEmpty(this.getChildren())) {
			this.getChildren().parallelStream().forEach(planElement -> result.addAll(planElement.getAllCostObjects()));
		}
		return result;
	}

	public long getAmountTotal() {
		ArrayList<CostObject> result = new ArrayList<>(this.getCostObjects());
		if(CollectionUtils.isNotEmpty(this.getChildren())) {
			this.getChildren().parallelStream().forEach(planElement -> result.addAll(planElement.getAllCostObjects()));
		}
		return result.stream().mapToLong(CostObject::getAmount).sum();
	}

	public List<Element> getChildren() {
		if(children == null) {
			this.children = new ArrayList<>();
		}
		return this.children;
	}

	public void setParent(Element parent) {
		if(parent != null && this.parent != null) {
			if(!parent.equals(this.parent)) {
				this.parent.getChildren().remove(this);
				parent.getChildren().add(this);
			}
		} else if(parent == null && this.parent != null) {
			this.parent.getChildren().remove(this);
		} else if(parent != null) {
			parent.getChildren().add(this);
		}
		this.parent = parent;
	}

	public void setType(ElementType type) {
		this.type = type;
	}
}