package org.playground.base.models.superClasses;

import javax.persistence.*;

@MappedSuperclass
@SequenceGenerator(name = "seq", initialValue = 1, allocationSize = 100)
public abstract class BaseElement {

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "seq")
	private Long id;

	@Basic(optional = false)
	@Column(nullable = false)
	private long createdTimeMillis;

	@Basic(optional = false)
	@Column(nullable = false)
	private long updatedTimeMillis;

	@Version
	private long version;

	@Basic
	private boolean deleted;

	//
	// --- model methods ---
	//

	@Override
	public int hashCode() {
		if(id == null) {
			//			throw new IllegalStateException("hashCode() before persisting");
			return super.hashCode();
		}
		return id.hashCode();
	}

	@Override
	public boolean equals(Object that) {
		if(id == null) {
			return super.equals(that);
		}
		if(this == that) {
			return true;
		}
		if(that instanceof BaseElement) {
			return id.equals(((BaseElement) that).id);
		}

		return false;
	}

	public Long getId() {
		return id;
	}

	public long getCreatedTimeMillis() {
		return createdTimeMillis;
	}

	public void setCreatedTimeMillis(long createdTimeMillis) {
		this.createdTimeMillis = createdTimeMillis;
	}

	public long getUpdatedTimeMillis() {
		return updatedTimeMillis;
	}

	public void setUpdatedTimeMillis(long updatedTimeMillis) {
		this.updatedTimeMillis = updatedTimeMillis;
	}

	public long getVersion() {
		return version;
	}

	public boolean isDeleted() {
		return deleted;
	}

	public void setDeleted(boolean deleted) {
		this.deleted = deleted;
	}
}