package org.playground.crud.dao;

import org.playground.base.TenantEntityManagerProvider;
import org.playground.base.models.Element;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;

@Component
public class PlanElementDao {

	private final TenantEntityManagerProvider tenantEntityManagerProvider;

	public PlanElementDao(TenantEntityManagerProvider tenantEntityManagerProvider) {
		this.tenantEntityManagerProvider = tenantEntityManagerProvider;
	}

	public Element create(Element element) {
		element.setCreatedTimeMillis(System.currentTimeMillis());
		this.tenantEntityManagerProvider.getEntityManager().persist(element);
		return element;
	}

	public List<Element> findByType(Long typeId) {
		return this.tenantEntityManagerProvider.getEntityManager()
		                                       .createQuery(
				                                       "SELECT pe FROM Element pe WHERE pe.type is not null AND pe.type.id = :typeId ORDER BY pe.createdTimeMillis",
				                                       Element.class)
		                                       .setParameter("typeId", typeId)
		                                       .getResultList();
	}

	public List<Element> deleteAll() {
		List<Element> all = this.tenantEntityManagerProvider.getEntityManager()
		                                                    .createQuery("SELECT pe FROM Element pe ORDER BY pe.createdTimeMillis",
		                                                                     Element.class)
		                                                    .getResultList();
		if(!CollectionUtils.isEmpty(all)) {
			for(Element element : all) {
				this.tenantEntityManagerProvider.getEntityManager().remove(element);
			}
		}

		return all;
	}
}