package org.playground;

import org.playground.TestApplication;
import org.playground.base.TenantEntityManagerProvider;
import org.playground.base.models.Element;
import org.playground.base.models.ElementType;
import org.playground.base.models.embeddable.CostObject;
import org.playground.base.models.enums.Type;
import org.playground.crud.services.interfaces.ElementService;
import org.playground.crud.services.interfaces.ElementTypeService;
import org.junit.jupiter.api.*;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.ArrayList;
import java.util.List;

@ExtendWith(SpringExtension.class)
@SpringBootTest(classes = TestApplication.class)
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class ErrorTest {

	@Autowired
	protected ElementService elementService;
	@Autowired
	protected ElementTypeService elementTypeService;

	ElementType mainType;

	@BeforeAll
	public void reset(@Autowired ElementService elementService, @Autowired ElementTypeService elementTypeService) {
		elementTypeService.deleteAll();
		elementService.deleteAll();

		this.mainType = this.elementTypeService.create(ElementType.of(Type.MAIN, "PLAN"));
		ElementType subType = this.elementTypeService.create(ElementType.of(Type.SUB, "SUB"));

		for(int i = 0; i < 2000; i++) {
			Element element = Element.of(mainType, null);
			element.addCostObject(CostObject.of(2));
			element.addCostObject(CostObject.of(3));
			element.addCostObject(CostObject.of(4));
			element = this.elementService.create(element);

			for(int j = 0; j < 5; j++) {
				Element childElement = Element.of(subType, element);
				childElement.addCostObject(CostObject.of(2));
				childElement.addCostObject(CostObject.of(3));
				childElement.addCostObject(CostObject.of(4));
				this.elementService.create(childElement);
			}
		}
	}

	@AfterAll
	public void reset(@Autowired TenantEntityManagerProvider tenantEntityManagerProvider) {
		tenantEntityManagerProvider.closeAll();
	}

	/**
	 * This test always returns a different size for the costObjects list, but never the correct one
	 */
	@Test
	public void loadCostObjectOfAllElementsIncludingChildren() {
		List<Element> allElements = this.elementService.findByType(mainType.getId());

		ArrayList<CostObject> costObjects = new ArrayList<>();
		for(Element element : allElements) {
			costObjects.addAll(element.getAllCostObjects());
		}
		Assertions.assertEquals(36000, costObjects.size());
	}

	/**
	 * This test sometimes throws a NullPointerException, sometimes it just has a wrong totalAmount
	 */
	@Test
	public void loadAmountOfAllElementsIncludingChildren() {
		List<Element> allElements = this.elementService.findByType(mainType.getId());

		long totalAmount = 0;
		for(Element element : allElements) {
			totalAmount += element.getAmountTotal();
		}
		Assertions.assertEquals(108000, totalAmount);
	}
}
